
{ simple MAPI definitions                                                  }
{ Version 1.00.00 :                                                        }
{   16 bit mapi.h partially converted by Keith Darragh, (c) 1995 SWIFTSOFT }
{ Version 2.00.00 :                                                        }
{   extended and adapted to 32 bit by Danny Heijl, (c) 1996  CEVI VZW      }
{   also put in a separate "header" file, "smapi.pas"                      }
{ Version 2.00.0a :                                                        }
{   added DeletMail for completeness, Danny Heijl Dec 1996                 }
{ Version 2.00.0b :                                                        }
{   added "Address" (select addresses), Rudy Claasen, Jan 1997             }
{   added '.DLL' to 'MAPI32' for Windows NT users, Danny Heijl, Jan 1997   }
{                                                                          }
{ Danny.Heijl@cevi.be                                                      }

{ Version 2.01.00 :                                                        }
{ Changes by Stefan Hoffmeister (Stefan.Hoffmeister@Uni-Passau.de):
  - rework the implementation part to allow dynamic linking
  - add a function (MAPIavailable) that indicates whether MAPI services are
    available.
}

unit smapi;

{$F+}

interface

uses SysUtils;

type
{$IFDEF WIN32}
  SString   = ShortString;
{$ELSE}
  SString   = string;
{$ENDIF}

  EMAPIdllerror = Class(Exception);

{------------------}
{ MAPI data types. }
{------------------}

  UINT      =  Integer;
  ULONG     =  LongInt;
  lpULONG   = ^LongInt;
  LHANDLE   =  LongInt;
  lpLHANDLE = ^LHANDLE;
  DWORD     =  Longint;
  LPVOID    =  Pointer;


{-----------------------------------}
{ MAPI File attachment information. }
{-----------------------------------}

type
  lpMapiFileDesc = ^TMapiFileDesc;
  TMapiFileDesc = Record
    ulReserved : ULONG;                  { Reserved for future use. Must be 0 }
    flFlags    : ULONG;     		         { Flags }
    nPosition : ULONG;                   { Attachment position in message body }
    lpszPathName : PChar;                { Full path name of attachment file }
    lpszFileName : PChar;                { Original filename (optional) }
    lpFileType : Pointer;                { Attachment file type (reserved) }
  end;

{-----------------------------}
{ MAPI recipient information. }
{-----------------------------}

type
  lpMapiRecipDesc = ^TMapiRecipDesc;
  lppMapiRecipDesc = ^lpMapiRecipDesc;
  TMapiRecipDesc = Record
    ulReserved   : ULONG;                { Reserved for future use. Must be 0 }
    ulRecipClass : ULONG;                { Recipient class - }
                                         { MAPI_TO, MAPI_CC, }
                                         { MAPI_BCC, MAPI_ORIG }
    lpszName    : PChar;                 { Recipient name }
    lpszAddress : PChar;                 { Recipient address (optional) }
    ulEIDSize   : ULONG;                 { Size (in bytes) of lpEntryID }
    lpEntryID   : Pointer;               { System-specific recipient reference }
  end;

{---------------------------}
{ MAPI message information. }
{---------------------------}

type
  lpMapiMessage = ^TMapiMessage;
  TlppMapiMessage = ^lpMapiMessage;
  TMapiMessage = Record
    ulReserved         : ULONG;           { Reserved for future use. Must be 0 }
    lpszSubject        : PChar;           { Message subject }
    lpszNoteText       : PChar;           { Message text }
    lpszMessageType    : PChar;           { Message class }
    lpszDateReceived   : PChar;           { In YYYY/MM/DD HH:MM format }
    lpszConversationID : PChar;	          { Conversation thread ID }
    flFlags            : ULONG;           { Unread, return receipt }
    lpOriginator       : lpMapiRecipDesc; { Originator descriptor }
    nRecipCount        : ULONG;           { Number of recipients }
    lpRecips           : lpMapiRecipDesc; { Recipient descriptors }
    nFileCount         : ULONG;           { Number of file attachments }
    lpFiles            : lpMapiFileDesc;  { Attachment descriptors }
  end;

{-----------------------------}
{ MAPI constants information. }
{-----------------------------}

Const
  MAPI_ORIG                     =  0;    { Recipient is message originator. }
  MAPI_TO                       =  1;    { Recipient is primary recipient. }
  MAPI_CC                       =  2;    { Recipient is copy recipient. }
  MAPI_BCC                      =  3;    { Recipient is blind-copy recipient. }

  MAPI_UNREAD            :ULONG = $0001;
  MAPI_RECEIPT_REQUESTED :ULONG = $0002;
  MAPI_SENT              :ULONG = $0004;

  MAPI_LOGON_UI          :ULONG = $0001; { Display logon UI }
  MAPI_NEW_SESSION       :ULONG = $0002; { Don't get default if available. }
  MAPI_DIALOG            :ULONG = $0008; { Prompt to resolve ambig. names }
  MAPI_UNREAD_ONLY       :ULONG = $0020; { Only unread messages }
  MAPI_ENVELOPE_ONLY     :ULONG = $0040; { Only header information }
  MAPI_PEEK              :ULONG = $0080; { Don't mark message as read }
  MAPI_GUARANTEE_FIFO    :ULONG = $0100; { Guarantee FIFO MAPIFindNext }
  MAPI_BODY_AS_FILE      :ULONG = $0200; { Save body as first attachment }
  MAPI_AB_NOMODIFY       :ULONG = $0400; { Don't modify PAB entries }
  MAPI_SUPPRESS_ATTACH   :ULONG = $0800; { Header and body, no files }
  MAPI_FORCE_DOWNLOAD    :ULONG = $1000; { Force message download from server }

  MAPI_OLE               :ULONG = $0001; { Set if attachment is an OLE Object }
  MAPI_OLE_STATIC        :ULONG = $0002; { Set if attachment is Static OLE    }

{ MAPI Error codes. }

const
  SUCCESS_SUCCESS                 =  0;
  MAPI_USER_ABORT                 =  1;
  MAPI_E_FAILURE                  =  2;
  MAPI_E_LOGIN_FAILURE            =  3;
  MAPI_E_DISK_FULL                =  4;
  MAPI_E_INSUFFICIENT_MEMORY      =  5;
  MAPI_E_ACCESS_DENIED		        =  6;
  MAPI_E_TOO_MANY_SESSIONS        =  8;
  MAPI_E_TOO_MANY_FILES           =  9;
  MAPI_E_TOO_MANY_RECIPIENTS      = 10;
  MAPI_E_ATTACHMENT_NOT_FOUND     = 11;
  MAPI_E_ATTACHMENT_OPEN_FAILURE  = 12;
  MAPI_E_ATTACHMENT_WRITE_FAILURE = 13;
  MAPI_E_UNKNOWN_RECIPIENT        = 14;
  MAPI_E_BAD_RECIPTYPE            = 15;
  MAPI_E_NO_MESSAGES              = 16;
  MAPI_E_INVALID_MESSAGE          = 17;
  MAPI_E_TEXT_TOO_LARGE           = 18;
  MAPI_E_INVALID_SESSION	        = 19;
  MAPI_E_TYPE_NOT_SUPPORTED	     = 20;
  MAPI_E_AMBIGUOUS_RECIPIENT	     = 21;
  MAPI_E_MESSAGE_IN_USE		     = 22;
  MAPI_E_NETWORK_FAILURE	        = 23;
  MAPI_E_INVALID_EDITFIELDS	     = 24;
  MAPI_E_INVALID_RECIPS		     = 25;
  MAPI_E_NOT_SUPPORTED		        = 26;


{ Declare calls to MAPI.DLL }

Function MAPIFindNext(lhSession        : LHANDLE;
                      ulUIParam        : ULONG;
                      lpszMessageType  : PChar;
                      lpszSeedMessageID: PChar;
                      flFlags          : ULONG;
                      ulReserved       : ULONG;
                      lpszMessageID    : PChar) :  ULONG ;

Function MAPILogoff(lhSession  : LHANDLE;
                    ulUIParam  : ULONG;
                    flFlags    : ULONG;
                    ulReserved : ULONG) : ULONG ;

Function MAPILogon(ulUIParam   : ULONG;
                   lpszName    : PChar;
                   lpszPassword: PChar;
                   flFlags     : ULONG;
                   ulReserved  : ULONG;
                   lplhSession : LPLHANDLE) : ULONG;

Function MAPISendMail(lhSession  : LHANDLE;
                      ulUIParam  : ULONG;
                      lpMessage  : lpMapiMessage;
                      flFlags    : ULONG;
                      ulReserved : ULONG) : ULONG;

Function MAPIReadMail(lhSession     : LHANDLE;
                      ulUIParam     : ULONG;
                      lpszMessageID : PChar;
                      flFlags       : ULONG;
                      ulReserved    : ULONG;
                      lpMessage     : lpMapiMessage) : ULONG;

Function MAPIDeleteMail(lhSession     : LHANDLE;
                        ulUIParam     : ULONG;
                        lpszMessageID : PChar;
                        flFlags       : ULONG;
                        ulReserved    : ULONG) : ULONG;


Function MAPIResolveName(lhSession : LHANDLE;
                         ulUIParam : ULONG;
                         lpszName  : Pchar;
                         flFlags   : ULONG;
                         ulReserved: ULONG;
                         lppRecips : lppMapiRecipDesc) : ULONG;

Function MAPIFreeBuffer(pv : LPVOID) : ULONG;

Function MAPIAddress(lhSession: LHANDLE;
                      ulUIParam: ULONG;
                      lpszCaption: PChar;
                      nEditFields: ULONG;
                      lpszLabels: PChar;
                      nRecips: ULONG;
                      var lpRecips: TMapiRecipDesc;
                      flFlags: ULONG;
                      ulReserved: ULONG;
                      lpnNewRecips: lpULONG;
                      var lppNewRecips: lpMapiRecipDesc): ULONG;



 Function MAPIavailable: boolean;


implementation

uses
{$IFDEF Win32}
  Windows;
{$ELSE}
  WinProcs, WinTypes;
{$ENDIF}

var
  DLLinitialized: boolean; { MAPI there? }
  DLLHandle : THandle;     { Handle of MAPI.DLL }

const
{$IFDEF Win32}
  DLLName = 'MAPI32.DLL';
{$ELSE}
  DLLName = 'MAPI.DLL';
{$ENDIF}

{ declare the MAPI dll function types }
type
TFNMAPIFindNext = Function (lhSession  : LHANDLE;
                      ulUIParam        : ULONG;
                      lpszMessageType  : PChar;
                      lpszSeedMessageID: PChar;
                      flFlags          : ULONG;
                      ulReserved       : ULONG;
                      lpszMessageID    : PChar) :  ULONG
{$IFNDEF WIN32} ; {$ELSE} stdcall; {$ENDIF}

TFNMAPILogoff = Function(lhSession  : LHANDLE;
                    ulUIParam  : ULONG;
                    flFlags    : ULONG;
                    ulReserved : ULONG) : ULONG
{$IFNDEF WIN32} ; {$ELSE} stdcall; {$ENDIF}

TFNMAPILogon = Function(ulUIParam   : ULONG;
                   lpszName    : PChar;
                   lpszPassword: PChar;
                   flFlags     : ULONG;
                   ulReserved  : ULONG;
                   lplhSession : LPLHANDLE) : ULONG
{$IFNDEF WIN32} ; {$ELSE} stdcall; {$ENDIF}

TFNMAPISendMail = Function(lhSession  : LHANDLE;
                      ulUIParam  : ULONG;
                      lpMessage  : lpMapiMessage;
                      flFlags    : ULONG;
                      ulReserved : ULONG) : ULONG
{$IFNDEF WIN32} ; {$ELSE} stdcall; {$ENDIF}

TFNMAPIReadMail = Function(lhSession     : LHANDLE;
                      ulUIParam     : ULONG;
                      lpszMessageID : PChar;
                      flFlags       : ULONG;
                      ulReserved    : ULONG;
                      lpMessage     : lpMapiMessage) : ULONG
{$IFNDEF WIN32} ; {$ELSE} stdcall; {$ENDIF}

TFNMAPIDeleteMail = Function(lhSession     : LHANDLE;
                        ulUIParam     : ULONG;
                        lpszMessageID : PChar;
                        flFlags       : ULONG;
                        ulReserved    : ULONG) : ULONG
{$IFNDEF WIN32} ; {$ELSE} stdcall; {$ENDIF}


TFNMAPIResolveName = Function(lhSession : LHANDLE;
                         ulUIParam : ULONG;
                         lpszName  : Pchar;
                         flFlags   : ULONG;
                         ulReserved: ULONG;
                         lppRecips : lppMapiRecipDesc) : ULONG
{$IFNDEF WIN32} ; {$ELSE} stdcall; {$ENDIF}

TFNMAPIFreeBuffer = Function(pv : LPVOID) : ULONG
{$IFNDEF WIN32} ;{$ELSE} stdcall; {$ENDIF}

TFNMAPIAddress = Function(lhSession: LHANDLE;
                      ulUIParam: ULONG;
                      lpszCaption: PChar;
                      nEditFields: ULONG;
                      lpszLabels: PChar;
                      nRecips: ULONG;
                      var lpRecips: TMapiRecipDesc;
                      flFlags: ULONG;
                      ulReserved: ULONG;
                      lpnNewRecips: lpULONG;
                      var lppNewRecips: lpMapiRecipDesc): ULONG
{$IFNDEF WIN32} ;{$ELSE} stdcall; {$ENDIF}


{ declare the functions themselves }
var
  FNMAPIFindNext:    TFNMAPIFindNext;
  FNMAPILogoff:      TFNMAPILogoff;
  FNMAPILogon:       TFNMAPILogon;
  FNMAPISendMail:    TFNMAPISendMail;
  FNMAPIReadMail:    TFNMAPIReadMail;
  FNMAPIDeleteMail:  TFNMAPIDeleteMail;
  FNMAPIResolveName: TFNMAPIResolveName;
  FNMAPIFreeBuffer:  TFNMAPIFreeBuffer;
  FNMAPIAddress:     TFNMAPIAddress;


{ define the local version of these functions  }
{ they simply call the MAPI dll functions      }

Function MAPIFindNext(lhSession        : LHANDLE;
                      ulUIParam        : ULONG;
                      lpszMessageType  : PChar;
                      lpszSeedMessageID: PChar;
                      flFlags          : ULONG;
                      ulReserved       : ULONG;
                      lpszMessageID    : PChar) :  ULONG;
begin
  Result := FNMAPIFindNext( lhSession,ulUIParam,lpszMessageType,lpszSeedMessageID,
                            flFlags,ulReserved,lpszMessageID);
end;

Function MAPILogoff(lhSession  : LHANDLE;
                    ulUIParam  : ULONG;
                    flFlags    : ULONG;
                    ulReserved : ULONG) : ULONG;
begin
  Result := FNMAPILogoff( lhSession,ulUIParam,flFlags,ulReserved);
end;

Function MAPILogon(ulUIParam   : ULONG;
                   lpszName    : PChar;
                   lpszPassword: PChar;
                   flFlags     : ULONG;
                   ulReserved  : ULONG;
                   lplhSession : LPLHANDLE) : ULONG;
begin
  Result := FNMAPILogon( ulUIParam,lpszName,lpszPassword,flFlags,ulReserved,
                         lplhSession);
end;

Function MAPISendMail(lhSession  : LHANDLE;
                      ulUIParam  : ULONG;
                      lpMessage  : lpMapiMessage;
                      flFlags    : ULONG;
                      ulReserved : ULONG) : ULONG;
begin
  Result := FNMAPISendMail( lhSession,ulUIParam,lpMessage,flFlags,
                            ulReserved);
end;

Function MAPIReadMail(lhSession     : LHANDLE;
                      ulUIParam     : ULONG;
                      lpszMessageID : PChar;
                      flFlags       : ULONG;
                      ulReserved    : ULONG;
                      lpMessage     : lpMapiMessage) : ULONG;
begin
  Result := FNMAPIReadMail( lhSession,ulUIParam,lpszMessageID,flFlags,
                            ulReserved,lpMessage);
end;

Function MAPIDeleteMail(lhSession     : LHANDLE;
                        ulUIParam     : ULONG;
                        lpszMessageID : PChar;
                        flFlags       : ULONG;
                        ulReserved    : ULONG) : ULONG;
begin
  Result := FNMAPIDeleteMail( lhSession,ulUIParam,lpszMessageID,flFlags,
                              ulReserved);
end;


Function MAPIResolveName(lhSession : LHANDLE;
                         ulUIParam : ULONG;
                         lpszName  : Pchar;
                         flFlags   : ULONG;
                         ulReserved: ULONG;
                         lppRecips : lppMapiRecipDesc) : ULONG;
begin
  Result := FNMAPIResolveName( lhSession,ulUIParam,lpszName,flFlags,
                               ulReserved,lppRecips);
end;

Function MAPIFreeBuffer(pv : LPVOID) : ULONG;
begin
  Result := FNMAPIFreeBuffer(pv);
end;

Function MAPIAddress(lhSession: LHANDLE;
                      ulUIParam: ULONG;
                      lpszCaption: PChar;
                      nEditFields: ULONG;
                      lpszLabels: PChar;
                      nRecips: ULONG;
                      var lpRecips: TMapiRecipDesc;
                      flFlags: ULONG;
                      ulReserved: ULONG;
                      lpnNewRecips: lpULONG;
                      var lppNewRecips: lpMapiRecipDesc): ULONG;
begin
  Result := FNMAPIAddress( lhSession,ulUIParam,lpszCaption,nEditFields,
                           lpszLabels,nRecips,lpRecips,flFlags,
                           ulReserved,lpnNewRecips,lppNewRecips);
end;



Function MAPIavailable: boolean;
begin
  MAPIavailable := DLLinitialized;
end;


Procedure InitializeSMAPI;
begin
  DLLHandle := LoadLibrary(DLLName);
  if DLLHandle <= HInstance_Error then
  begin
    FreeLibrary(DLLHandle);
  end
  else
  begin
    DLLinitialized := true;

    @FNMAPIFindNext:=   GetProcAddress(DLLHandle, 'MAPIFindNext');
    @FNMAPILogoff:=     GetProcAddress(DLLHandle, 'MAPILogoff');
    @FNMAPILogon:=      GetProcAddress(DLLHandle, 'MAPILogon');
    @FNMAPISendMail:=   GetProcAddress(DLLHandle, 'MAPISendMail');
    @FNMAPIReadMail:=   GetProcAddress(DLLHandle, 'MAPIReadMail');
    @FNMAPIDeleteMail:= GetProcAddress(DLLHandle, 'MAPIDeleteMail');
    @FNMAPIResolveName:=GetProcAddress(DLLHandle, 'MAPIResolveName');
    @FNMAPIFreeBuffer:= GetProcAddress(DLLHandle, 'MAPIFreeBuffer');
    @FNMAPIAddress:=    GetProcAddress(DLLHandle, 'MAPIAddress');
    if    (@FNMAPIAddress     = Nil)
       or (@FNMAPIFreeBuffer  = Nil)
       or (@FNMAPIResolveName = Nil)
       or (@FNMAPIDeleteMail  = Nil)
       or (@FNMAPIReadMail    = Nil)
       or (@FNMAPISendMail    = Nil)
       or (@FNMAPILogon       = Nil)
       or (@FNMAPILogoff      = Nil)
       or (@FNMAPIFindNext    = Nil) then
         raise EMAPIdllerror.Create('GetProcAddress of MAPI DLL function(s) fails');
  end;
end;

{ this should be not necessary... }
procedure ClearProcedureAdresses;
begin
  @FNMAPIFindNext:=   nil;
  @FNMAPILogoff:=     nil;
  @FNMAPILogon:=      nil;
  @FNMAPISendMail:=   nil;
  @FNMAPIReadMail:=   nil;
  @FNMAPIDeleteMail:= nil;
  @FNMAPIResolveName:=nil;
  @FNMAPIFreeBuffer:= nil;
  @FNMAPIAddress:=    nil;
end;


procedure FinalizeSMAPI; far; { explicitly make it far - perhaps F+ vanishes...}
begin
  if DLLinitialized then
    FreeLibrary(DLLHandle);

  ClearProcedureAdresses;
end;


initialization
  ClearProcedureAdresses; { just for safety }

  DLLinitialized := false;
  DLLHandle := 0;

{$IFNDEF Win32}
  AddExitProc(FinalizeSMAPI);
{$ENDIF}

  InitializeSMAPI;

{$IFDEF Win32}
finalization
  FinalizeSMAPI;
{$ENDIF}
end.
